/*

IFT1169 - Gestion de la mémoire - Exemple 02 -

09-décembre	- 2017 v1.04	M.Lokbani - GCC 7.2 avec -std=c++17
14-septembre	- 2016 v1.03	M.Lokbani - GCC 6.1.1 avec -std=c++11
09-mars			- 2015 v1.02	M.Lokbani - GCC 4.9.2  avec -std=c++11
09-novembre	- 2013 v1.01	M.Lokbani - GCC 4.8.1  avec -std=c++11
18-novembre	- 2003 v1.00	M.Lokbani - Extrait des notes de cours.


contact: lokbani@iro.umontreal.ca

        
    Copyright (C) 1999-2017 Université de Montréal
            Département d'informatique et de Recherche Opérationnelle
            Mohamed Lokbani
    -- Tous Droits Réservés -- All Rights Reserved --

Description:
------------

    Gestion de la mémoire
    
        Cet exemple montre qu'il est interdit de modifier un emplacement constant
        
    Voir notes de cours, 3.1:
    
*/

#include <iostream>

using namespace std;

int main() {

#ifdef CSCHAR
	const char *texte1 = "Bonjour";
#endif
	
	char *texte2 = "Un autre bonjour";

#ifdef CSCHAR
	texte1[1] = 'a';    // Erreur de syntaxe
#endif

	texte2[1] = 'a';    // Peut provoquer une erreur lors de l'execution du programme

	return 0;
}


/*

	-1- Si on compile en ajoutant l'option -DCSCHAR, le compilateur anticipe le problème de la ligne 38
	et signale un avertissement. Comme il y a une initialisation, il préfère avoir aussi que le contenu ne
	change pas en cours de route.

	>g++ -Os -Wall -pedantic -DCSCHAR -c 02mem.cpp -o 02mem.o
	02mem.cpp: In function ‘int main()’:
	02mem.cpp:40:17: warning: ISO C++ forbids converting a string constant to ‘char*’ [-Wpedantic]
		char *texte2 = "Un autre bonjour";
				 ^~~~~~~~~~~~~~~~~~
	02mem.cpp:43:14: error: assignment of read-only location ‘*(texte1 + 1u)’
		texte1[1] = 'a';    // Erreur de syntaxe


	-2- On compile sans l'option -DCSCHAR, il signale un avertissement car nous avons inclus l'option
	pedantic, sinon c'est lettre à la poste!

	>g++ -Wall -pedantic -Os -c 02mem.cpp -o 02mem.o
	
	02mem.cpp: In function ‘int main()’:
	02mem.cpp:40:17: warning: ISO C++ forbids converting a string constant to ‘char*’ [-Wpedantic]
	char *texte2 = "Un autre bonjour";
			 ^

	>Exit code: 0

	Et Si on exécute le programme: il va planter.

	./02mem 
		Segmentation fault

*/