/*

IFT1169 - Gestion de la mémoire - Exemple 12 -

09-décembre	- 2017 v1.03	M.Lokbani - GCC 7.2 avec -std=c++17
09-mars			- 2015 v1.02	M.Lokbani - GCC 4.9.2  avec -std=c++11
07-novembre	- 2013 v1.01	M.Lokbani - GCC 4.8.1  avec -std=c++11
18-novembre	- 2003 v1.00	M.Lokbani - Extrait des notes de cours.



contact: lokbani@iro.umontreal.ca

        
    Copyright (C) 1999-2017 Université de Montréal
            Département d'informatique et de Recherche Opérationnelle
            Mohamed Lokbani
    -- Tous Droits Réservés -- All Rights Reserved --

Description:
------------

    Gestion de la mémoire
    
        Utilisation de l'opérateur "new" et définition d'une fonction à appeler
        en cas d'échec lors de l'allocation de la mémoire
        
    Voir notes de cours, 5.1:
    
    Le try/catch n'a servi à rien car dès qu'une exception a été levée c'est la fonction
    "myNewHandler" qui a pris les choses en main.
    
*/

#include <iostream>
#include <string>
#include <cstdlib>

using namespace std;

void myNewHandler(){
	cerr << "Tentative d'allocation mémoire à échouer!" << endl;
	abort();
}

    
void f(int size) {
	char *p = new char [size];
	if (p == 0){
		cout << "Plus de mémoire libre!"<<endl;
	}
	return;
}

const int BUF_SIZE = 999999999;

int main(){
	set_new_handler(myNewHandler);          // Installer  "new handler"

	int *pn = new int;                // un seul entier
	*pn = 10;
	int *pi = new int(5);             // un seul entier, valeur 5
	double *pd = new double(55.9);   // un double, valeur 55.9
	int *buf = new int[10];           // pointeur sur 10 entiers

	cout << "pn: " << *pn << " ; pi: " << *pi << " ; pd: " << *pd << " ; buf[0]: " << buf[0] << endl;

	for (int i=0;i<100;i++){
		cout << "i: " << i << endl;
		try {
			f(BUF_SIZE);
		}catch( bad_alloc &ba) {
			cout << ba.what( ) << endl;
			exit (EXIT_FAILURE);
		}
	}

	return 0;
}

/*

	La dernière valeur de i va dépendre de la m.émoire disponible sur votre machine.

	>05mem
	pn: 10 ; pi: 5 ; pd: 55.9 ; buf[0]: 5767512
	i: 0
	i: 1
	i: 2
	i: 3
	i: 4
	i: 5
	i: 6
	i: 7
	i: 8
	i: 9
	i: 10
	i: 11
	i: 12
	i: 13
	i: 14
	i: 15
	i: 16
	i: 17
	i: 18
	i: 19
	i: 20
	i: 21
	i: 22
	i: 23
	i: 24
	i: 25
	i: 26
	i: 27
	i: 28
	i: 29
	i: 30
	Tentative d'allocation mémoire à échouer!

	This application has requested the Runtime to terminate it in an unusual way.
	Please contact the application's support team for more information.
	>Exit code: 3

*/
